import controlP5.*;

ControlP5 controlP5;


long res = 10000000;
CastManager cm;

long savedFrames = 0;
boolean record=false;
int fr = 30000;

int controlW = 125;
int controlH = 8;
int controlX = 10;
int controlY = 10;
int controlSpace = 10;

float nDiv = 0.02;
void setup() {
  size(800,600);
  controlP5 = new ControlP5(this);
  background(0); 
  cm = new CastManager();
  cm.start();
  
  // Set up the UI controls
  controlP5.addSlider("rIntensity", 0, 100, cm.cR.intensity, controlX, cY(0), controlW, controlH);
  controlP5.addSlider("gIntensity", 0, 100, cm.cG.intensity, controlX, cY(1), controlW, controlH);
  controlP5.addSlider("bIntensity", 0, 100, cm.cB.intensity, controlX, cY(2), controlW, controlH);
  
  controlP5.addSlider("noiseScale", 0, 50, cm.noiseScale*1500, controlX, cY(5), controlW, controlH);
  
  controlP5.addSlider("boxW", 0,  width, cm.boxW, controlX, cY(7), controlW, controlH);
  controlP5.addSlider("boxH", 0, height, cm.boxH, controlX, cY(8), controlW, controlH);
  
  Radio r = controlP5.addRadio("emitMode", controlX, cY(10));
  r.add("spiral", 0);
  r.add("spiralclear", 1);
  r.add("random", 2);
  
  controlP5.addSlider("depth", 1,200, cm.cR.d, controlX, cY(15), controlW, controlH);
  
  controlP5.addSlider("ior", 1, 3, cm.cR.n, controlX, cY(17), controlW, controlH);
  controlP5.addSlider("iorDivergence", 0, 0.2, cm.cG.n-cm.cR.n, controlX, cY(18), controlW, controlH);
  
  controlP5.addButton("clear", 0, controlX, cY(20), controlW/2-2, controlH);
  controlP5.addButton("reset", 0, controlX+controlW/2+2, cY(20), controlW/2-2, controlH).setLabel("reseed");
  frameRate(fr);
}

// What y position should this controller be at?
int cY(int n) {
  return controlY + n*controlSpace; 
}

void depth(float d) {
  cm.cR.d = cm.cG.d = cm.cB.d = d;
  clear();
}

void ior(float n) {
  cm.cR.n = n;
  cm.cG.n = n+nDiv;
  cm.cB.n = n+nDiv*2;
  clear();
}
void iorDivergence(float d) {
  nDiv = d;
  ior(cm.cR.n);
}


void emitMode(int theID) {
  switch (theID) {
    case 0:
      cm.algo = cm.SPIRAL_ALGO;
      break;
    case 1:
      cm.algo = cm.SPIRAL_CLEAR_ALGO;
      break;
    case 2:
      cm.algo = cm.RANDOM_ALGO;
      break;
  }
}
void noiseScale(float s) {
  cm.noiseScale = s/1500.0;
  clear();
}
void rIntensity(float i) {
  cm.cR.intensity = i;
  clear();
}
void gIntensity(float i) {
  cm.cG.intensity = i;
  clear();
}
void bIntensity(float i) {
  cm.cB.intensity = i;
  clear();
}
void boxW(float w) {
  cm.boxW = w;
}
void boxH(float h) {
  cm.boxH = h;
}

void reset() {
      cm.reseed();
      cm.reset();
}
void clear() {
  cm.reset();
}
void keyPressed() {
  switch (key) {
    case 'r':  //reset
      reset();
      break;
    case 'c': //clear
      clear();
      break;
    case 'p':
      println("LoopCount: " + cm.loopCount);
      println("fps: " + cm.fps);
      break;
    case 's':
      controlP5.hide();
      saveFrame("SAVED"+millis()+".png");
      break;
    case 'm':
      record=!record;
      break;
    case 'a':
      cm.cycleAlgo();
      break;
    case 'f':
      if (fr > 1000) fr=2;
      else fr = 3000;
      frameRate(fr);
      break;
    }
}


void draw() {
  controlP5.show();
  if (cm.algo == cm.SPIRAL_CLEAR_ALGO) {
    clear();
  }
  loadPixels();
  float sum = 0;
  float maxV = 0;
  float count=0;
  for (int i = 0; i < cm.cR.bg.length; i++) {
    float m = max(cm.cR.bg[i],cm.cG.bg[i], cm.cB.bg[i]);
    sum+=m;
    maxV = maxV>m?maxV:m;
    if (m!=0) count++;
  }
  float avg = sum/count;
  
  for (int i = 0; i < pixels.length; i++) {
    pixels[i] = color(cm.cR.bg[i]*128/avg, cm.cG.bg[i]*128/avg,cm.cB.bg[i]*128/avg);
    if (keyPressed && key=='n') pixels[i] = color(noise(i%width*cm.noiseScale, i/width*cm.noiseScale, cm.z*cm.noiseScale)*255);
  }
  updatePixels();
  
  if (record && cm.loopCount > res) {
    cm.loopCount = 0;
    cm.z+=0.8;
    cm.clear();
    saveFrame("f-" + savedFrames++ + ".png");
    background(0);
  }
}
